
#include "Viewer2D.h"

#include <GL/glut.h>
#include <math.h>

Viewer2D::Viewer2D( QWidget *parent, const char *name ) :
   QGLWidget( parent, name ), _dir( 0 ),
   _dirBeingDragged( 0 ), _fileBeingDragged( 0 )
{ }

void Viewer2D::initializeGL() {
   glClearColor( 0.0, 0.0, 0.0, 0.0 );
   //glDepthFunc( GL_LEQUAL );
   glDisable( GL_DEPTH_TEST );
   glDisable( GL_LIGHTING );
}

void Viewer2D::resizeGL( int w, int h ) {
   glViewport( 0, 0, w-1, h-1 );
   _widthInPixels = w;
   _heightInPixels = h;
}

void drawSquare( const Point2D& p, const char * string ) {
   glBegin( GL_LINE_LOOP );
      glVertex2f( p.x-1, p.y-1 );
      glVertex2f( p.x+1, p.y-1 );
      glVertex2f( p.x+1, p.y+1 );
      glVertex2f( p.x-1, p.y+1 );
   glEnd();
   glPushMatrix();
      glTranslatef( p.x-1, p.y-2, 0 );
      glScalef( 1.0/150, 1.0/150, 1.0/150 );
      for ( int i = 0; string[i] != '\0'; ++i )
         glutStrokeCharacter( GLUT_STROKE_MONO_ROMAN, string[i] );
   glPopMatrix();
}

void Viewer2D::paintGL() {
   //glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glClear( GL_COLOR_BUFFER_BIT );

   if ( 0 != _dir ) {

      glMatrixMode( GL_PROJECTION );
      glLoadIdentity();
      Rect2D r;
      r.set( _r, _widthInPixels/(float)_heightInPixels );
      glOrtho( r.x, r.x+r.w, r.y, r.y+r.h, -1, 1 );

      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();

      glColor3f( 0, 1, 1 );
      list< Directory* >::const_iterator dirIt = _childDirs.begin();
      for ( ; dirIt != _childDirs.end(); ++dirIt ) {
         drawSquare( (*dirIt)->getPoint(), (*dirIt)->getName().latin1() );
      }
      glColor3f( 1, 1, 1 );
      list< FileItem* >::const_iterator fileIt = _childFiles.begin();
      for ( ; fileIt != _childFiles.end(); ++fileIt ) {
         drawSquare( (*fileIt)->getPoint(), (*fileIt)->text(0).latin1() );
      }
   }
   swapBuffers();
}

void Viewer2D::mousePressEvent( QMouseEvent * e ) {
   _mouseX = e->x();
   _mouseY = e->y();
   _dirBeingDragged = 0;
   _fileBeingDragged = 0;
   if ( (e->button() & Qt::LeftButton) && !(e->state() & Qt::MidButton) ) {
      Rect2D r;
      r.set( _r, _widthInPixels/(float)_heightInPixels );
      Point2D p
         = r.pixelToPoint(_mouseX,_mouseY,_widthInPixels,_heightInPixels);

      list< Directory* >::const_iterator dirIt = _childDirs.begin();
      Point2D q;
      for ( ; dirIt != _childDirs.end(); ++dirIt ) {
         q = (*dirIt)->getPoint();
         if ( q.x-1 <= p.x && p.x <= q.x+1 && q.y-1 <= p.y && p.y <= q.y+1 ) {
            _dirBeingDragged = (*dirIt);
            return;
         }
      }
      list< FileItem* >::const_iterator fileIt = _childFiles.begin();
      for ( ; fileIt != _childFiles.end(); ++fileIt ) {
         q = (*fileIt)->getPoint();
         if ( q.x-1 <= p.x && p.x <= q.x+1 && q.y-1 <= p.y && p.y <= q.y+1 ) {
            _fileBeingDragged = (*fileIt);
            return;
         }
      }
   }
}

void Viewer2D::mouseReleaseEvent( QMouseEvent * ) {
}

void Viewer2D::mouseMoveEvent( QMouseEvent * e ) {
   int newx = e->x();
   int newy = e->y();
   Rect2D r;
   r.set( _r, _widthInPixels/(float)_heightInPixels );
   if ( e->state() & Qt::LeftButton ) {
      if ( e->state() & Qt::MidButton ) {
         // zoom
         int delta = ( newx - _mouseX ) + ( _mouseY - newy );
         float factor = pow( 1.01, -0.5*delta );
         Point2D centre( r.x + r.w/2, r.y + r.h/2 );
         r.w *= factor;
         r.h *= factor;
         r.x = centre.x - r.w/2;
         r.y = centre.y - r.h/2;
         _dir->getRect() = _r = r;
         repaint();
      }
      else if ( 0 != _dirBeingDragged || 0 != _fileBeingDragged ) {
         // move object
         Point2D p0, p1;
         p0 = r.pixelToPoint(_mouseX,_mouseY,_widthInPixels,_heightInPixels);
         p1 = r.pixelToPoint( newx, newy, _widthInPixels, _heightInPixels );
         if ( 0 != _dirBeingDragged ) {
            Point2D P = _dirBeingDragged->getPoint();
            _dirBeingDragged->setPoint(Point2D(P.x+p1.x-p0.x, P.y+p1.y-p0.y));
         }
         else {
            Point2D P = _fileBeingDragged->getPoint();
            _fileBeingDragged->setPoint(Point2D(P.x+p1.x-p0.x, P.y+p1.y-p0.y));
         }
         repaint();
      }
   }
   else if ( e->state() & Qt::MidButton ) {
      // pan
      Point2D p0, p1;
      p0 = r.pixelToPoint(_mouseX,_mouseY,_widthInPixels,_heightInPixels);
      p1 = r.pixelToPoint( newx, newy, _widthInPixels, _heightInPixels );
      r.x -= p1.x - p0.x;
      r.y -= p1.y - p0.y;
      _dir->getRect() = _r = r;
      repaint();
   }
   _mouseX = newx;
   _mouseY = newy;
}

void Viewer2D::mouseDoubleClickEvent( QMouseEvent * ) {
   //puts("viewer: mouse double click");
}

void Viewer2D::dragMoveEvent( QDragMoveEvent * ) {
   //puts("viewer: mouse drag");
}

void Viewer2D::fileSelected( QListViewItem * item ) {
   _dir = dynamic_cast< Directory* >( item );
   _childDirs.clear();
   _childFiles.clear();
   if ( 0 != _dir ) {
      QListViewItem * child = _dir->firstChild();
      if ( 0 == child ) {
         _dir->readContents();
         // try again ...
         child = _dir->firstChild();
      }
      while ( 0 != child ) {
         if ( Directory* d = dynamic_cast< Directory* >( child ) ) {
            _childDirs.push_back( d );
         }
         else if ( FileItem* f = dynamic_cast< FileItem* >( child ) ) {
            _childFiles.push_back( f );
         }
         child = child->nextSibling();
      }
      _r = _dir->getRect();
   }
   repaint();
}

void Viewer2D::clear() {
   _dir = 0;
   _childDirs.clear();
   _childFiles.clear();
   repaint();
}

